@@ -2,7 +2,20 @@
  * @file syscall_rmo.c
  *
  * Copyright (C) 2005 IBM Corporation
- * Common Public License Version 1.0 (see COPYRIGHT)
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * @author John Rose <johnrose@us.ibm.com>
  */
@@ -11,13 +24,13 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <sys/mman.h>
-#include "common.h"
+#include <inttypes.h>
 #include "syscall.h"
 #include "librtas.h"
 
 #define MAX_PAGES 64
+#define MAX_PATH_LEN 80
 
 struct workarea_config {
 	int init_done;
@@ -36,6 +49,36 @@ static struct workarea_config wa_config = {
 	.pages_map = 0ll,
 };
 
+/**
+ * open_proc_rtas_file
+ * @brief Open the proc rtas file
+ *
+ * @param name filename to open
+ * @param mode mode to open file in
+ * @return results of open() call
+ */
+static int open_proc_rtas_file(const char *name, int mode)
+{
+	const char *proc_rtas_paths[] = { "/proc/ppc64/rtas", "/proc/rtas" };
+	char full_name[MAX_PATH_LEN];
+	int npaths;
+	int fd;
+	int i;
+
+	npaths = sizeof(proc_rtas_paths) / sizeof(char *);
+	for (i = 0; i < npaths; i++) {
+		sprintf(full_name, "%s/%s", proc_rtas_paths[i], name);
+		fd = open(full_name, mode, S_IRUSR | S_IWUSR);
+		if (fd >= 0)
+			break;
+	}
+
+	if (fd < 0)
+		dbg("Failed to open %s\n", full_name);
+
+	return fd;
+}
+
 /**
  * read_kregion_bounds
  * @brief Read the kernel region bounds for RMO memory
@@ -51,7 +94,7 @@ static int read_kregion_bounds(struct region *kregion)
 
 	fd = open_proc_rtas_file(rmo_filename, O_RDONLY);
 	if (fd < 0) {
-		dbg1("Could not open workarea file\n");
+		dbg("Could not open workarea file\n");
 		return RTAS_IO_ASSERT;
 	}
 
@@ -62,12 +105,12 @@ static int read_kregion_bounds(struct region *kregion)
 		return rc;
 	}
 
-	sscanf(buf, "%llx %x", &kregion->addr, &kregion->size);
+	sscanf(buf, "%" SCNx64 " %x", &kregion->addr, &kregion->size);
 	free(buf);
 
 	if (!(kregion->size && kregion->addr) ||
 	    (kregion->size > (PAGE_SIZE * MAX_PAGES))) {
-		dbg1("Unexpected kregion bounds\n");
+		dbg("Unexpected kregion bounds\n");
 		return RTAS_IO_ASSERT;
 	}
 
@@ -100,7 +143,7 @@ static inline uint64_t get_bits(short lobit, short hibit, uint64_t mask)
  * @return 0 on success, !0 otherwise
  */
 static inline void set_bits(short lobit, short hibit, uint64_t value,
-			    uint64_t * mask)
+			    uint64_t *mask)
 {
 	short num_bits = hibit - lobit + 1;
 	uint64_t ones_mask = (1ll << (num_bits)) - 1;
@@ -111,7 +154,7 @@ static inline void set_bits(short lobit, short hibit, uint64_t value,
 
 /**
  * acquire_file_lock
- * 
+ *
  * @param start
  * @param size
  * @return 0 on success, !0 otherwise
@@ -126,7 +169,7 @@ static int acquire_file_lock(off_t start, size_t size)
 		wa_config.lockfile_fd = open(lockfile_path, O_CREAT | O_RDWR,
 					     S_IRUSR | S_IWUSR);
 		if (wa_config.lockfile_fd < 0) {
-			dbg1("could not open lockfile %s\n", lockfile_path);
+			dbg("could not open lockfile %s\n", lockfile_path);
 			return RTAS_IO_ASSERT;
 		}
 	}
@@ -140,7 +183,7 @@ static int acquire_file_lock(off_t start, size_t size)
 	rc = fcntl(wa_config.lockfile_fd, F_SETLKW, &flock);
 	if (rc < 0) {
 		/* Expected to fail for regions used by other processes */
-		dbg1("fcntl failed for [0x%lx, 0x%x]\n", start, size);
+		dbg("fcntl failed for [0x%lx, 0x%zx]\n", start, size);
 		return RTAS_IO_ASSERT;
 	}
 
@@ -167,7 +210,7 @@ static int release_file_lock(off_t start, size_t size)
 
 	rc = fcntl(wa_config.lockfile_fd, F_SETLK, &flock);
 	if (rc < 0) {
-		dbg1("fcntl failed for [0x%lx, 0x%x]\n", start, size);
+		dbg("fcntl failed for [0x%lx, 0x%zx]\n", start, size);
 		return RTAS_IO_ASSERT;
 	}
 
@@ -190,7 +233,7 @@ static int get_phys_region(size_t size, uint32_t * phys_addr)
 	int i;
 
 	if (size > kregion->size) {
-		dbg1("Invalid buffer size 0x%x requested\n", size);
+		dbg("Invalid buffer size 0x%zx requested\n", size);
 		return RTAS_IO_ASSERT;
 	}
 
@@ -213,7 +256,7 @@ static int get_phys_region(size_t size, uint32_t * phys_addr)
 	}
 
 	if (!addr) {
-		dbg1("Could not find available workarea space\n");
+		dbg("Could not find available workarea space\n");
 		return RTAS_IO_ASSERT;
 	}
 
@@ -237,17 +280,17 @@ static int release_phys_region(uint32_t phys_addr, size_t size)
 	int rc;
 
 	if (size > kregion->size) {
-		dbg1("Invalid buffer size 0x%x requested\n", size);
+		dbg("Invalid buffer size 0x%zx requested\n", size);
 		return RTAS_IO_ASSERT;
 	}
 
 	first_page = (phys_addr - kregion->addr) / PAGE_SIZE;
 	n_pages = size / PAGE_SIZE;
 
-	bits =
-	    get_bits(first_page, first_page + n_pages - 1, wa_config.pages_map);
+	bits = get_bits(first_page, first_page + n_pages - 1,
+			wa_config.pages_map);
 	if (bits != ((1 << n_pages) - 1)) {
-		dbg1("Invalid region [0x%x, 0x%x]\n", phys_addr, size);
+		dbg("Invalid region [0x%x, 0x%zx]\n", phys_addr, size);
 		return RTAS_IO_ASSERT;
 	}
 
@@ -260,7 +303,7 @@ static int release_phys_region(uint32_t phys_addr, size_t size)
 
 /**
  * init_workarea_config
- * 
+ *
  * @return 0 on success, !0 otherwise
  */
 static int init_workarea_config()
@@ -292,7 +335,7 @@ static int mmap_dev_mem(uint32_t phys_addr, size_t size, void **buf)
 
 	fd = open(devmem_path, O_RDWR);
 	if (fd < 0) {
-		dbg1("Failed to open %s\n", devmem_path);
+		dbg("Failed to open %s\n", devmem_path);
 		return RTAS_IO_ASSERT;
 	}
 
@@ -301,7 +344,7 @@ static int mmap_dev_mem(uint32_t phys_addr, size_t size, void **buf)
 	close(fd);
 
 	if (newbuf == MAP_FAILED) {
-		dbg1("mmap failed\n");
+		dbg("mmap failed\n");
 		return RTAS_IO_ASSERT;
 	}
 
@@ -323,14 +366,14 @@ static int munmap_dev_mem(void *buf, size_t size)
 
 	fd = open(devmem_path, O_RDWR);
 	if (fd < 0) {
-		dbg1("Failed to open %s\n", devmem_path);
+		dbg("Failed to open %s\n", devmem_path);
 		return RTAS_IO_ASSERT;
 	}
 
 	rc = munmap(buf, size);
 	close(fd);
 	if (rc < 0) {
-		dbg1("munmap failed\n");
+		dbg("munmap failed\n");
 		return RTAS_IO_ASSERT;
 	}
 
@@ -338,11 +381,11 @@ static int munmap_dev_mem(void *buf, size_t size)
 }
 
 /**
- * sc_interface_exists
+ * interface_exists
  *
  * @return 0 on success, !0 otherwise
  */
-int sc_interface_exists()
+int interface_exists()
 {
 	int fd = open_proc_rtas_file(rmo_filename, O_RDONLY);
 	int exists;
@@ -356,7 +399,8 @@ int sc_interface_exists()
 }
 
 /**
- * sc_free_rmo_buffer
+ * rtas_free_rmo_buffer
+ * @brief free the rmo buffer used by librtas
  *
  * @param buf virtual address of mmap()'ed buffer
  * @param phys_addr physical address of low mem buffer
@@ -365,11 +409,15 @@ int sc_interface_exists()
  *	RTAS_FREE_ERR - Free called before get
  * 	RTAS_IO_ASSERT - Unexpected I/O Error
  */
-int sc_free_rmo_buffer(void *buf, uint32_t phys_addr, size_t size)
+int rtas_free_rmo_buffer(void *buf, uint32_t phys_addr, size_t size)
 {
 	int n_pages;
 	int rc;
 
+	rc = sanity_check();
+	if (rc)
+		return rc;
+
 	n_pages = size / PAGE_SIZE;
 
 	/* Check for multiple of page size */
@@ -380,13 +428,15 @@ int sc_free_rmo_buffer(void *buf, uint32_t phys_addr, size_t size)
 	}
 
 	if (!wa_config.init_done) {
-		dbg1("Attempting to free before calling get()\n");
+		dbg("Attempting to free before calling get()\n");
 		return RTAS_FREE_ERR;
 	}
 
 	rc = munmap_dev_mem(buf, size);
-	if (rc)
+	if (rc) {
+		(void) release_phys_region(phys_addr, size);
 		return rc;
+	}
 
 	rc = release_phys_region(phys_addr, size);
 
@@ -394,7 +444,12 @@ int sc_free_rmo_buffer(void *buf, uint32_t phys_addr, size_t size)
 }
 
 /**
- * sc_get_rmo_buffer
+ * rtas_get_rmo_buffer
+ * @brief Retrive the RMO buffer used by librtas
+ *
+ * On successful completion the buf parameter will reference an allocated
+ * area of RMO memory and the phys_addr parameter will refernce the
+ * physical address of the RMO buffer.
  *
  * @param size Size of requested region.  Must be a multiple of 4096.
  * @param buf Assigned to mmap'ed buffer of acquired region
@@ -404,13 +459,17 @@ int sc_free_rmo_buffer(void *buf, uint32_t phys_addr, size_t size)
  * 	RTAS_NO_LOWMEM - Out of rmo memory
  * 	RTAS_IO_ASSERT - Unexpected I/O Error
  */
-int sc_get_rmo_buffer(size_t size, void **buf, uint32_t * phys_addr)
+int rtas_get_rmo_buffer(size_t size, void **buf, uint32_t * phys_addr)
 {
 	uint32_t addr;
 	int n_pages;
 	int rc;
 
-	dbg1("RMO buffer request, size: %d\n", size);
+	rc = sanity_check();
+	if (rc)
+		return rc;
+
+	dbg("RMO buffer request, size: %zd\n", size);
 
 	n_pages = size / PAGE_SIZE;
 
@@ -432,8 +491,10 @@ int sc_get_rmo_buffer(size_t size, void **buf, uint32_t * phys_addr)
 		return rc;
 
 	rc = mmap_dev_mem(addr, size, buf);
-	if (rc)
+	if (rc) {
+		(void) release_phys_region(addr, size);
 		return rc;
+	}
 
 	*phys_addr = addr;
 	return 0;
