@@ -3,7 +3,20 @@
  * @brief RTAS SRC/FRU sections routines
  *
  * Copyright (C) 2005 IBM Corporation
- * Common Public License Version 1.0 (see COPYRIGHT)
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * @author Nathan Fontenot <nfont@austin.ibm.com>
  */
@@ -194,6 +207,7 @@ parse_src_scn(struct rtas_event *re)
 
     src_raw = malloc(sizeof(*src_raw));
     if (src_raw == NULL) {
+	free(src);
         errno = ENOMEM;
         return 1;
     }
